"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseCommand = void 0;
/* eslint-disable import/no-dynamic-require */
/* eslint-disable global-require */
/* eslint-disable @typescript-eslint/no-var-requires */
const core_1 = require("@oclif/core");
const path_1 = __importDefault(require("path"));
const cosmiconfig_1 = require("cosmiconfig");
// import { PlayerLanguageService } from '@player-ui/language-service';
const dsl_1 = require("@player-ui/dsl");
const configLoader = (0, cosmiconfig_1.cosmiconfig)('player');
/** The common configs for all  */
class BaseCommand extends core_1.Command {
    loadConfig(configFilePath) {
        return __awaiter(this, void 0, void 0, function* () {
            if (configFilePath) {
                try {
                    return yield configLoader.load(configFilePath);
                }
                catch (e) {
                    this.warn(`Error loading config file: ${configFilePath}`);
                }
            }
            return configLoader.search();
        });
    }
    resolveConfig(conf, relativePath) {
        var _a, _b, _c, _d;
        return __awaiter(this, void 0, void 0, function* () {
            let config = Object.assign(Object.assign({}, (conf !== null && conf !== void 0 ? conf : {})), { plugins: [] });
            // If there's an extension load it
            if (conf === null || conf === void 0 ? void 0 : conf.extends) {
                let normalizedExtension;
                if (typeof conf.extends === 'string') {
                    const requiredExtendedConfig = require(conf.extends);
                    normalizedExtension =
                        (_a = requiredExtendedConfig.default) !== null && _a !== void 0 ? _a : requiredExtendedConfig;
                }
                else {
                    normalizedExtension = conf.extends;
                }
                config = Object.assign({}, (yield this.resolveConfig(normalizedExtension, relativePath)));
            }
            yield Promise.all((_c = (_b = conf === null || conf === void 0 ? void 0 : conf.presets) === null || _b === void 0 ? void 0 : _b.map((preset) => __awaiter(this, void 0, void 0, function* () {
                var _e;
                if (typeof preset === 'string') {
                    const requiredExtendedConfig = require(preset);
                    const normalizedExtension = (_e = requiredExtendedConfig.default) !== null && _e !== void 0 ? _e : requiredExtendedConfig;
                    const extendedConfig = yield this.resolveConfig(normalizedExtension);
                    config.plugins = [...extendedConfig.plugins, ...config.plugins];
                    return;
                }
                const presetConfig = yield this.resolveConfig(preset);
                config.plugins = [...presetConfig.plugins, ...config.plugins];
            }))) !== null && _c !== void 0 ? _c : []);
            // Go through each plugin and load/create it
            (_d = conf === null || conf === void 0 ? void 0 : conf.plugins) === null || _d === void 0 ? void 0 : _d.forEach((pluginInfo) => {
                var _a;
                if (typeof pluginInfo === 'object' && !Array.isArray(pluginInfo)) {
                    config.plugins.push(pluginInfo);
                    return;
                }
                const pluginName = typeof pluginInfo === 'string' ? pluginInfo : pluginInfo[0];
                const pluginArgs = typeof pluginInfo === 'string' ? undefined : pluginInfo[1];
                let pluginLoadPath = pluginName;
                if (pluginName.startsWith('.')) {
                    pluginLoadPath = path_1.default.resolve(relativePath !== null && relativePath !== void 0 ? relativePath : '', pluginName);
                }
                this.debug('loading plugin from %s', pluginLoadPath);
                // Get the instance for the plugin
                const required = require(pluginLoadPath);
                const PluginExport = (_a = required.default) !== null && _a !== void 0 ? _a : required;
                if (!PluginExport) {
                    return;
                }
                const pluginInstance = typeof PluginExport === 'object'
                    ? PluginExport
                    : new PluginExport(pluginArgs);
                config.plugins.push(pluginInstance);
            });
            return config;
        });
    }
    readConfig() {
        return __awaiter(this, void 0, void 0, function* () {
            const { flags } = yield this.parse();
            const configFile = yield this.loadConfig(flags.config);
            return this.resolveConfig(configFile === null || configFile === void 0 ? void 0 : configFile.config);
        });
    }
    getPlayerConfig() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.resolvedConfig) {
                return this.resolvedConfig;
            }
            const c = yield this.readConfig();
            this.resolvedConfig = c;
            return c;
        });
    }
    // async createLanguageService(): Promise<PlayerLanguageService> {
    //   const lsp = new PlayerLanguageService();
    //   const { plugins } = await this.getPlayerConfig();
    //   for (let i = 0; i < plugins.length; i++) {
    //     // eslint-disable-next-line no-await-in-loop
    //     await plugins[i].onCreateLanguageService?.(lsp);
    //   }
    //   return lsp;
    // }
    createDSLCompiler() {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            const compiler = new dsl_1.DSLCompiler();
            const { plugins } = yield this.getPlayerConfig();
            for (let i = 0; i < plugins.length; i++) {
                // eslint-disable-next-line no-await-in-loop
                yield ((_b = (_a = plugins[i]).onCreateDSLCompiler) === null || _b === void 0 ? void 0 : _b.call(_a, compiler));
            }
            return compiler;
        });
    }
    exit(code) {
        if (process.env.NODE_ENV !== 'test') {
            super.exit(code);
        }
    }
}
exports.BaseCommand = BaseCommand;
BaseCommand.flags = {
    config: core_1.Flags.string({
        description: 'Path to a specific config file to load.\nBy default, will automatically search for an rc or config file to load',
        char: 'c',
    }),
};
